package swingtables.logic;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;

public class DataSource {

    private Timer pageInfoLoaderTimer;

    private DataSource() {
    }

    public static DataSource getInstance() {
        return DataSourceHolder.INSTANCE;
    }

    private static class DataSourceHolder {

        private static final DataSource INSTANCE = new DataSource();
    }
    private List<WikipediaPageInfo> wikipediaPageInfos = new LinkedList<>();

    public List<WikipediaPageInfo> getWikipediaPageInfos() {
        return wikipediaPageInfos;
    }

    public void startLoading() {
        pageInfoLoaderTimer = new Timer();
        pageInfoLoaderTimer.schedule(new WikipediaPageInfoLoader() {
            @Override
            public void pageInfoLoaded(WikipediaPageInfo wikipediaPageInfo) {
                wikipediaPageInfos.add(wikipediaPageInfo);
                for (DataSourceListener dataSourceListener : dataSourceListeners) {
                    dataSourceListener.wikipediaPageInfoLoaded(wikipediaPageInfo);
                }
            }
        }, 1000, 1000);
    }

    public void stopLoading() {
        if (pageInfoLoaderTimer != null) {
            pageInfoLoaderTimer.cancel();
        }
    }
    private List<DataSourceListener> dataSourceListeners = new ArrayList<>();

    public void addDataSourceListener(DataSourceListener listener) {
        dataSourceListeners.add(listener);
    }

    public void removeDataSourceListener(DataSourceListener listener) {
        dataSourceListeners.remove(listener);
    }
}
