package swingtables.logic;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.sql.Date;
import java.util.Scanner;
import java.util.TimerTask;
import javax.xml.bind.DatatypeConverter;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public abstract class WikipediaPageInfoLoader extends TimerTask implements Runnable {

    @Override
    public void run() {
        try {
            String randomWikiPageAddress = "?";
            Socket socket;
            try {
                socket = new Socket("en.wikipedia.org", 80);
            } catch (IOException ex) {
                System.err.println(ex.getMessage());
                return;
            }
            try (PrintWriter requestWriter = new PrintWriter(socket.getOutputStream())) {
                requestWriter.println("GET /wiki/Special:Random HTTP/1.1\nHost: en.wikipedia.org\nConnection: close\n\n");
                requestWriter.flush();
                try (Scanner scanner = new Scanner(socket.getInputStream())) {
                    while (scanner.hasNextLine()) {
                        String string = scanner.nextLine();
                        if (string.startsWith("Location: ")) {
                            randomWikiPageAddress = string.split(" ")[1];
                        }
                    }
                }
            } catch (IOException ex) {
                System.err.println(ex.getMessage());
                return;
            } finally {
                if (randomWikiPageAddress.equals("?")) {
                    System.err.println("Cannot get random wiki page");
                    return;
                }
            }
            URL url;
            try {
                url = new URL("http://en.wikipedia.org/w/api.php?format=json&action=query&prop=info&inprop=url&titles=" + randomWikiPageAddress.replace("http://en.wikipedia.org/wiki/", ""));
            } catch (MalformedURLException ex) {
                System.err.println(ex.getMessage());
                return;
            }
            StringBuilder jsonTextBuilder = new StringBuilder();
            try (Scanner scanner = new Scanner(url.openStream())) {
                while (scanner.hasNextLine()) {
                    String string = scanner.nextLine();
                    jsonTextBuilder.append(string);
                }
            } catch (IOException ex) {
                System.err.println(ex.getMessage());
                return;
            }
            JSONObject jsonRoot = new JSONObject(jsonTextBuilder.toString());
            JSONObject jsonQuery = (JSONObject) jsonRoot.get("query");
            JSONObject jsonPages = (JSONObject) jsonQuery.get("pages");
            JSONArray jsonPagesArray = jsonPages.names();
            for (int i = 0; i < jsonPagesArray.length(); i++) {
                JSONObject jsonPage = jsonPages.getJSONObject(jsonPagesArray.getString(i));
                System.out.println(jsonPage.toString(2));
                WikipediaPageInfo wpi = new WikipediaPageInfo();
                wpi.setTitle(jsonPage.getString("title"));
                wpi.setLength(jsonPage.getLong("length"));
                wpi.setPageID(jsonPage.getLong("pageid"));
                try {
                    wpi.setUrl(new URL(jsonPage.getString("fullurl")));
                } catch (MalformedURLException ex) {
                    System.err.println(ex.getMessage());
                    return;
                }
                wpi.setTouched(new Date(DatatypeConverter.parseDateTime(jsonPage.getString("touched")).getTimeInMillis()));
                pageInfoLoaded(wpi);
            }
        } catch (JSONException ex) {
            ex.printStackTrace();
        }
    }

    public abstract void pageInfoLoaded(WikipediaPageInfo wikipediaPageInfo);
}
